/* ============================================================================
 * Copyright (c) 2010, Michael A. Jackson (BlueQuartz Software)
 * Copyright (c) 2010, Dr. Michael A. Groeber (US Air Force Research Laboratories
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this
 * list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *
 * Neither the name of Michael A. Groeber, Michael A. Jackson, the US Air Force,
 * BlueQuartz Software nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  This code was written under United States Air Force Contract number
 *                           FA8650-07-D-5800
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

#include "AppVersion.h"

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
AppVersion::AppVersion() :
m_MajorNum(-1),
m_MinorNum(-1),
m_PatchNum(-1)
{

}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
AppVersion::AppVersion(int majorNum, int minorNum, int patchNum)
{
  m_MajorNum = majorNum;
  m_MinorNum = minorNum;
  m_PatchNum = patchNum;
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
AppVersion::~AppVersion()
{

}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
AppVersion::AppVersion(const AppVersion& rhs)
{
  m_MajorNum = rhs.m_MajorNum;
  m_MinorNum = rhs.m_MinorNum;
  m_PatchNum = rhs.m_PatchNum;
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
void AppVersion::operator=(const AppVersion& rhs)
{
  m_MajorNum = rhs.m_MajorNum;
  m_MinorNum = rhs.m_MinorNum;
  m_PatchNum = rhs.m_PatchNum;
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
bool AppVersion::operator==(const AppVersion& rhs)
{
  return (m_MajorNum == rhs.m_MajorNum &&
      m_MinorNum == rhs.m_MinorNum &&
      m_PatchNum == rhs.m_PatchNum);
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
bool AppVersion::operator>(const AppVersion& rhs)
{
  if (m_MajorNum > rhs.m_MajorNum)
  {
    return true;
  }
  else if (m_MajorNum == rhs.m_MajorNum)
  {
    if (m_MinorNum > rhs.m_MinorNum)
    {
      return true;
    }
    else if (m_MinorNum == rhs.m_MinorNum)
    {
      if (m_PatchNum > rhs.m_PatchNum)
      {
        return true;
      }
      else
      {
        return false;
      }
    }
    else
    {
      return false;
    }
  }
  else
  {
    return false;
  }
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
bool AppVersion::operator<(const AppVersion& rhs)
{
  if (m_MajorNum < rhs.m_MajorNum)
  {
    return true;
  }
  else if (m_MajorNum == rhs.m_MajorNum)
  {
    if (m_MinorNum < rhs.m_MinorNum)
    {
      return true;
    }
    else if (m_MinorNum == rhs.m_MinorNum)
    {
      if (m_PatchNum < rhs.m_PatchNum)
      {
        return true;
      }
      else
      {
        return false;
      }
    }
    else
    {
      return false;
    }
  }
  else
  {
    return false;
  }
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
int AppVersion::getMajorNum()
{
  return m_MajorNum;
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
int AppVersion::getMinorNum()
{
  return m_MinorNum;
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
int AppVersion::getPatchNum()
{
  return m_PatchNum;
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
void AppVersion::setMajorNum(int major)
{
  m_MajorNum = major;
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
void AppVersion::setMinorNum(int minor)
{
  m_MinorNum = minor;
}

// -----------------------------------------------------------------------------
//
// -----------------------------------------------------------------------------
void AppVersion::setPatchNum(int patch)
{
  m_PatchNum = patch;
}
